@@ -33,6 +33,7 @@
 #include "../PrecompiledHeaders.h"
 #include "Font.h"
 
+#include "../SystemToolbox.h"
 #include "../Toolbox.h"
 #include "../OrthancException.h"
 
@@ -135,7 +136,7 @@ namespace Orthanc
   void Font::LoadFromFile(const std::string& path)
   {
     std::string font;
-    Toolbox::ReadFile(font, path);
+    SystemToolbox::ReadFile(font, path);
     LoadFromMemory(font);
   }
 
