@@ -35,9 +35,12 @@
 
 #include "../OrthancException.h"
 #include "../Logging.h"
-
 #include "JpegErrorManager.h"
 
+#if ORTHANC_SANDBOXED == 0
+#  include "../SystemToolbox.h"
+#endif
+
 #include <stdlib.h>
 #include <vector>
 
@@ -102,7 +105,7 @@ namespace Orthanc
 
   void JpegWriter::SetQuality(uint8_t quality)
   {
-    if (quality <= 0 || quality > 100)
+    if (quality == 0 || quality > 100)
     {
       throw OrthancException(ErrorCode_ParameterOutOfRange);
     }
@@ -111,6 +114,7 @@ namespace Orthanc
   }
 
 
+#if ORTHANC_SANDBOXED == 0
   void JpegWriter::WriteToFileInternal(const std::string& filename,
                                        unsigned int width,
                                        unsigned int height,
@@ -118,7 +122,7 @@ namespace Orthanc
                                        PixelFormat format,
                                        const void* buffer)
   {
-    FILE* fp = Toolbox::OpenFile(filename, FileMode_WriteBinary);
+    FILE* fp = SystemToolbox::OpenFile(filename, FileMode_WriteBinary);
     if (fp == NULL)
     {
       throw OrthancException(ErrorCode_CannotWriteFile);
@@ -154,6 +158,7 @@ namespace Orthanc
 
     fclose(fp);
   }
+#endif
 
 
   void JpegWriter::WriteToMemoryInternal(std::string& jpeg,
