@@ -34,6 +34,9 @@
 
 #include "LuaContext.h"
 
+#include "../DicomFormat/DicomArray.h"
+#include "../DicomFormat/DicomMap.h"
+
 #include <json/json.h>
 
 namespace Orthanc
@@ -64,6 +67,10 @@ namespace Orthanc
 
     void PushStringMap(const std::map<std::string, std::string>& value);
 
+    void PushDicom(const DicomMap& dicom);
+
+    void PushDicom(const DicomArray& dicom);
+
     void Execute()
     {
       ExecuteInternal(0);
