@@ -35,6 +35,7 @@
 
 #include "../Logging.h"
 #include "../OrthancException.h"
+#include "../Toolbox.h"
 
 #include <boost/lexical_cast.hpp>
 
@@ -91,10 +92,10 @@ namespace Orthanc
 
     if (convertJsonToXml_)
     {
-#if ORTHANC_PUGIXML_ENABLED == 1
+#if ORTHANC_ENABLE_PUGIXML == 1
       std::string s;
       Toolbox::JsonToXml(s, value);
-      output_.SetContentType("application/xml");
+      output_.SetContentType("application/xml; charset=utf-8");
       output_.Answer(s);
 #else
       LOG(ERROR) << "Orthanc was compiled without XML support";
@@ -104,7 +105,7 @@ namespace Orthanc
     else
     {
       Json::StyledWriter writer;
-      output_.SetContentType("application/json");
+      output_.SetContentType("application/json; charset=utf-8");
       output_.Answer(writer.write(value));
     }
 
