@@ -1,7 +1,7 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2014 Medical Physics Department, CHU of Liege,
- * Belgium
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
+ * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -78,7 +78,7 @@ namespace Orthanc
     }
   }
 
-  bool RestApiPath::Match(HttpHandler::Arguments& components,
+  bool RestApiPath::Match(IHttpHandler::Arguments& components,
                           UriComponents& trailing,
                           const std::string& uriRaw) const
   {
@@ -87,7 +87,7 @@ namespace Orthanc
     return Match(components, trailing, uri);
   }
 
-  bool RestApiPath::Match(HttpHandler::Arguments& components,
+  bool RestApiPath::Match(IHttpHandler::Arguments& components,
                           UriComponents& trailing,
                           const UriComponents& uri) const
   {
@@ -135,7 +135,7 @@ namespace Orthanc
 
   bool RestApiPath::Match(const UriComponents& uri) const
   {
-    HttpHandler::Arguments components;
+    IHttpHandler::Arguments components;
     UriComponents trailing;
     return Match(components, trailing, uri);
   }
