@@ -1,7 +1,7 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2014 Medical Physics Department, CHU of Liege,
- * Belgium
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
+ * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,7 +33,7 @@
 #pragma once
 
 #include "../Toolbox.h"
-#include "../HttpServer/HttpHandler.h"
+#include "../HttpServer/IHttpHandler.h"
 
 #include <map>
 
@@ -50,11 +50,11 @@ namespace Orthanc
     RestApiPath(const std::string& uri);
 
     // This version is slower
-    bool Match(HttpHandler::Arguments& components,
+    bool Match(IHttpHandler::Arguments& components,
                UriComponents& trailing,
                const std::string& uriRaw) const;
 
-    bool Match(HttpHandler::Arguments& components,
+    bool Match(IHttpHandler::Arguments& components,
                UriComponents& trailing,
                const UriComponents& uri) const;
 
