@@ -38,6 +38,7 @@
 #include "../Core/Logging.h"
 
 #include <dcmtk/dcmdata/dcfilefo.h>
+#include <dcmtk/dcmdata/dcdeftag.h>
 
 
 namespace Orthanc
@@ -104,21 +105,15 @@ namespace Orthanc
     if (!summary_.HasContent())
     {
       summary_.Allocate();
-      FromDcmtkBridge::Convert(summary_.GetContent(), 
-                               *parsed_.GetContent().GetDcmtkObject().getDataset(),
-                               ORTHANC_MAXIMUM_TAG_LENGTH,                               
-                               Configuration::GetDefaultEncoding());
+      Configuration::ExtractDicomSummary(summary_.GetContent(), 
+                                         *parsed_.GetContent().GetDcmtkObject().getDataset());
     }
     
     if (!json_.HasContent())
     {
       json_.Allocate();
-      FromDcmtkBridge::ToJson(json_.GetContent(), 
-                              *parsed_.GetContent().GetDcmtkObject().getDataset(),
-                              DicomToJsonFormat_Full, 
-                              DicomToJsonFlags_Default,
-                              ORTHANC_MAXIMUM_TAG_LENGTH,
-                              Configuration::GetDefaultEncoding());
+      Configuration::ExtractDicomAsJson(json_.GetContent(), 
+                                        *parsed_.GetContent().GetDcmtkObject().getDataset());
     }
   }
 
@@ -276,4 +271,25 @@ namespace Orthanc
       return "";
     }
   }
+
+
+  bool DicomInstanceToStore::LookupTransferSyntax(std::string& result)
+  {
+    ComputeMissingInformation();
+
+    DicomMap header;
+    if (DicomMap::ParseDicomMetaInformation(header, GetBufferData(), GetBufferSize()))
+    {
+      const DicomValue* value = header.TestAndGetValue(DICOM_TAG_TRANSFER_SYNTAX_UID);
+      if (value != NULL &&
+          !value->IsBinary() &&
+          !value->IsNull())
+      {
+        result = Toolbox::StripSpaces(value->GetContent());
+        return true;
+      }
+    }
+
+    return false;
+  }
 }
