@@ -57,6 +57,8 @@ namespace Orthanc
     UidMap uidMap_;
     SetOfTags privateTagsToKeep_;
     bool allowManualIdentifiers_;
+    bool keepStudyInstanceUid_;
+    bool keepSeriesInstanceUid_;
 
     void MapDicomIdentifier(ParsedDicomFile& dicom,
                             ResourceType level);
