@@ -36,7 +36,6 @@
 #include "../../Core/Logging.h"
 #include "../../Core/OrthancException.h"
 #include "../../Core/Toolbox.h"
-#include "../../Core/Uuid.h"
 #include "../Internals/CommandDispatcher.h"
 #include "../OrthancInitialization.h"
 #include "EmbeddedResources.h"
@@ -97,7 +96,7 @@ namespace Orthanc
     worklistRequestHandlerFactory_ = NULL;
     applicationEntityFilter_ = NULL;
     checkCalledAet_ = true;
-    clientTimeout_ = 30;
+    associationTimeout_ = 30;
     continue_ = false;
   }
 
@@ -121,15 +120,18 @@ namespace Orthanc
     return port_;
   }
 
-  void DicomServer::SetClientTimeout(uint32_t timeout)
+  void DicomServer::SetAssociationTimeout(uint32_t seconds)
   {
+    LOG(INFO) << "Setting timeout for DICOM connections if Orthanc acts as SCP (server): " 
+              << seconds << " seconds (0 = no timeout)";
+
     Stop();
-    clientTimeout_ = timeout;
+    associationTimeout_ = seconds;
   }
 
-  uint32_t DicomServer::GetClientTimeout() const
+  uint32_t DicomServer::GetAssociationTimeout() const
   {
-    return clientTimeout_;
+    return associationTimeout_;
   }
 
 
