@@ -133,6 +133,9 @@ static const unsigned int MAXIMUM_STORAGE_SOP_CLASSES = 64;
 
 namespace Orthanc
 {
+  // By default, the timeout for DICOM SCU (client) connections is set to 10 seconds
+  static uint32_t defaultTimeout_ = 10;
+
   struct DicomUserConnection::PImpl
   {
     // Connection state
@@ -151,6 +154,7 @@ namespace Orthanc
 
     void Store(DcmInputStream& is, 
                DicomUserConnection& connection,
+               const std::string& moveOriginatorAET,
                uint16_t moveOriginatorID);
   };
 
@@ -256,6 +260,7 @@ namespace Orthanc
 
   void DicomUserConnection::PImpl::Store(DcmInputStream& is, 
                                          DicomUserConnection& connection,
+                                         const std::string& moveOriginatorAET,
                                          uint16_t moveOriginatorID)
   {
     CheckIsOpen();
@@ -277,21 +282,21 @@ namespace Orthanc
     const std::string syntax(xfer.getXferID());
     bool isGeneric = IsGenericTransferSyntax(syntax);
 
-    bool renegociate;
+    bool renegotiate;
     if (isGeneric)
     {
       // Are we making a generic-to-specific or specific-to-generic change of
       // the transfer syntax? If this is the case, renegotiate the connection.
-      renegociate = !IsGenericTransferSyntax(connection.GetPreferredTransferSyntax());
+      renegotiate = !IsGenericTransferSyntax(connection.GetPreferredTransferSyntax());
     }
     else
     {
-      // We are using a specific transfer syntax. Renegociate if the
+      // We are using a specific transfer syntax. Renegotiate if the
       // current connection does not match this transfer syntax.
-      renegociate = (syntax != connection.GetPreferredTransferSyntax());
+      renegotiate = (syntax != connection.GetPreferredTransferSyntax());
     }
 
-    if (renegociate)
+    if (renegotiate)
     {
       LOG(INFO) << "Change in the transfer syntax: the C-Store associated must be renegotiated";
 
@@ -338,12 +343,12 @@ namespace Orthanc
     request.DataSetType = DIMSE_DATASET_PRESENT;
     strncpy(request.AffectedSOPInstanceUID, sopInstance, DIC_UI_LEN);
 
-    strncpy(request.MoveOriginatorApplicationEntityTitle, 
-            connection.GetLocalApplicationEntityTitle().c_str(), DIC_AE_LEN);
-    request.opts = O_STORE_MOVEORIGINATORAETITLE;
-
-    if (moveOriginatorID != 0)
+    if (!moveOriginatorAET.empty())
     {
+      strncpy(request.MoveOriginatorApplicationEntityTitle, 
+              moveOriginatorAET.c_str(), DIC_AE_LEN);
+      request.opts = O_STORE_MOVEORIGINATORAETITLE;
+
       request.MoveOriginatorID = moveOriginatorID;  // The type DIC_US is an alias for uint16_t
       request.opts |= O_STORE_MOVEORIGINATORID;
     }
@@ -395,9 +400,7 @@ namespace Orthanc
       else
       {
         DicomMap m;
-        FromDcmtkBridge::Convert(m, *responseIdentifiers, 
-                                 ORTHANC_MAXIMUM_TAG_LENGTH,
-                                 Configuration::GetDefaultEncoding());
+        Configuration::ExtractDicomSummary(m, *responseIdentifiers);
 
         if (!m.HasTag(DICOM_TAG_QUERY_RETRIEVE_LEVEL))
         {
@@ -477,8 +480,8 @@ namespace Orthanc
   }
 
 
-  static DcmDataset* ConvertQueryFields(const DicomMap& fields,
-                                        ModalityManufacturer manufacturer)
+  static ParsedDicomFile* ConvertQueryFields(const DicomMap& fields,
+                                             ModalityManufacturer manufacturer)
   {
     switch (manufacturer)
     {
@@ -512,11 +515,11 @@ namespace Orthanc
           }
         }
 
-        return ToDcmtkBridge::Convert(*fix);
+        return new ParsedDicomFile(*fix);
       }
 
       default:
-        return ToDcmtkBridge::Convert(fields);
+        return new ParsedDicomFile(fields);
     }
   }
 
@@ -576,7 +579,9 @@ namespace Orthanc
 
     CheckIsOpen();
 
-    std::auto_ptr<DcmDataset> dataset(ConvertQueryFields(fields, manufacturer_));
+    std::auto_ptr<ParsedDicomFile> query(ConvertQueryFields(fields, manufacturer_));
+    DcmDataset* dataset = query->GetDcmtkObject().getDataset();
+
     const char* clevel = NULL;
     const char* sopClass = NULL;
 
@@ -584,19 +589,19 @@ namespace Orthanc
     {
       case ResourceType_Patient:
         clevel = "PATIENT";
-        DU_putStringDOElement(dataset.get(), DcmTagKey(0x0008, 0x0052), "PATIENT");
+        DU_putStringDOElement(dataset, DcmTagKey(0x0008, 0x0052), "PATIENT");
         sopClass = UID_FINDPatientRootQueryRetrieveInformationModel;
         break;
 
       case ResourceType_Study:
         clevel = "STUDY";
-        DU_putStringDOElement(dataset.get(), DcmTagKey(0x0008, 0x0052), "STUDY");
+        DU_putStringDOElement(dataset, DcmTagKey(0x0008, 0x0052), "STUDY");
         sopClass = UID_FINDStudyRootQueryRetrieveInformationModel;
         break;
 
       case ResourceType_Series:
         clevel = "SERIES";
-        DU_putStringDOElement(dataset.get(), DcmTagKey(0x0008, 0x0052), "SERIES");
+        DU_putStringDOElement(dataset, DcmTagKey(0x0008, 0x0052), "SERIES");
         sopClass = UID_FINDStudyRootQueryRetrieveInformationModel;
         break;
 
@@ -608,11 +613,11 @@ namespace Orthanc
           // This is a particular case for ClearCanvas, thanks to Peter Somlo <peter.somlo@gmail.com>.
           // https://groups.google.com/d/msg/orthanc-users/j-6C3MAVwiw/iolB9hclom8J
           // http://www.clearcanvas.ca/Home/Community/OldForums/tabid/526/aff/11/aft/14670/afv/topic/Default.aspx
-          DU_putStringDOElement(dataset.get(), DcmTagKey(0x0008, 0x0052), "IMAGE");
+          DU_putStringDOElement(dataset, DcmTagKey(0x0008, 0x0052), "IMAGE");
         }
         else
         {
-          DU_putStringDOElement(dataset.get(), DcmTagKey(0x0008, 0x0052), "INSTANCE");
+          DU_putStringDOElement(dataset, DcmTagKey(0x0008, 0x0052), "INSTANCE");
         }
 
         sopClass = UID_FINDStudyRootQueryRetrieveInformationModel;
@@ -629,26 +634,26 @@ namespace Orthanc
       case ResourceType_Instance:
         // SOP Instance UID
         if (!fields.HasTag(0x0008, 0x0018))
-          DU_putStringDOElement(dataset.get(), DcmTagKey(0x0008, 0x0018), "");
+          DU_putStringDOElement(dataset, DcmTagKey(0x0008, 0x0018), "");
 
       case ResourceType_Series:
         // Series instance UID
         if (!fields.HasTag(0x0020, 0x000e))
-          DU_putStringDOElement(dataset.get(), DcmTagKey(0x0020, 0x000e), "");
+          DU_putStringDOElement(dataset, DcmTagKey(0x0020, 0x000e), "");
 
       case ResourceType_Study:
         // Accession number
         if (!fields.HasTag(0x0008, 0x0050))
-          DU_putStringDOElement(dataset.get(), DcmTagKey(0x0008, 0x0050), "");
+          DU_putStringDOElement(dataset, DcmTagKey(0x0008, 0x0050), "");
 
         // Study instance UID
         if (!fields.HasTag(0x0020, 0x000d))
-          DU_putStringDOElement(dataset.get(), DcmTagKey(0x0020, 0x000d), "");
+          DU_putStringDOElement(dataset, DcmTagKey(0x0020, 0x000d), "");
 
       case ResourceType_Patient:
         // Patient ID
         if (!fields.HasTag(0x0010, 0x0020))
-          DU_putStringDOElement(dataset.get(), DcmTagKey(0x0010, 0x0020), "");
+          DU_putStringDOElement(dataset, DcmTagKey(0x0010, 0x0020), "");
 
         break;
 
@@ -657,7 +662,7 @@ namespace Orthanc
     }
 
     assert(clevel != NULL && sopClass != NULL);
-    ExecuteFind(result, pimpl_->assoc_, dataset.get(), sopClass, false, clevel, pimpl_->dimseTimeout_);
+    ExecuteFind(result, pimpl_->assoc_, dataset, sopClass, false, clevel, pimpl_->dimseTimeout_);
   }
 
 
@@ -667,21 +672,22 @@ namespace Orthanc
   {
     CheckIsOpen();
 
-    std::auto_ptr<DcmDataset> dataset(ConvertQueryFields(fields, manufacturer_));
+    std::auto_ptr<ParsedDicomFile> query(ConvertQueryFields(fields, manufacturer_));
+    DcmDataset* dataset = query->GetDcmtkObject().getDataset();
 
     const char* sopClass = UID_MOVEStudyRootQueryRetrieveInformationModel;
     switch (level)
     {
       case ResourceType_Patient:
-        DU_putStringDOElement(dataset.get(), DcmTagKey(0x0008, 0x0052), "PATIENT");
+        DU_putStringDOElement(dataset, DcmTagKey(0x0008, 0x0052), "PATIENT");
         break;
 
       case ResourceType_Study:
-        DU_putStringDOElement(dataset.get(), DcmTagKey(0x0008, 0x0052), "STUDY");
+        DU_putStringDOElement(dataset, DcmTagKey(0x0008, 0x0052), "STUDY");
         break;
 
       case ResourceType_Series:
-        DU_putStringDOElement(dataset.get(), DcmTagKey(0x0008, 0x0052), "SERIES");
+        DU_putStringDOElement(dataset, DcmTagKey(0x0008, 0x0052), "SERIES");
         break;
 
       case ResourceType_Instance:
@@ -691,11 +697,11 @@ namespace Orthanc
           // This is a particular case for ClearCanvas, thanks to Peter Somlo <peter.somlo@gmail.com>.
           // https://groups.google.com/d/msg/orthanc-users/j-6C3MAVwiw/iolB9hclom8J
           // http://www.clearcanvas.ca/Home/Community/OldForums/tabid/526/aff/11/aft/14670/afv/topic/Default.aspx
-          DU_putStringDOElement(dataset.get(), DcmTagKey(0x0008, 0x0052), "IMAGE");
+          DU_putStringDOElement(dataset, DcmTagKey(0x0008, 0x0052), "IMAGE");
         }
         else
         {
-          DU_putStringDOElement(dataset.get(), DcmTagKey(0x0008, 0x0052), "INSTANCE");
+          DU_putStringDOElement(dataset, DcmTagKey(0x0008, 0x0052), "INSTANCE");
         }
         break;
 
@@ -721,7 +727,7 @@ namespace Orthanc
     T_DIMSE_C_MoveRSP response;
     DcmDataset* statusDetail = NULL;
     DcmDataset* responseIdentifiers = NULL;
-    OFCondition cond = DIMSE_moveUser(pimpl_->assoc_, presID, &request, dataset.get(),
+    OFCondition cond = DIMSE_moveUser(pimpl_->assoc_, presID, &request, dataset,
                                       NULL, NULL,
                                       /*opt_blockMode*/ DIMSE_BLOCKING, 
                                       /*opt_dimse_timeout*/ pimpl_->dimseTimeout_,
@@ -782,7 +788,7 @@ namespace Orthanc
     remotePort_ = 104;
     manufacturer_ = ModalityManufacturer_Generic;
 
-    SetTimeout(10); 
+    SetTimeout(defaultTimeout_);
     pimpl_->net_ = NULL;
     pimpl_->params_ = NULL;
     pimpl_->assoc_ = NULL;
@@ -957,6 +963,7 @@ namespace Orthanc
 
   void DicomUserConnection::Store(const char* buffer, 
                                   size_t size,
+                                  const std::string& moveOriginatorAET,
                                   uint16_t moveOriginatorID)
   {
     // Prepare an input stream for the memory buffer
@@ -965,24 +972,26 @@ namespace Orthanc
       is.setBuffer(buffer, size);
     is.setEos();
       
-    pimpl_->Store(is, *this, moveOriginatorID);
+    pimpl_->Store(is, *this, moveOriginatorAET, moveOriginatorID);
   }
 
   void DicomUserConnection::Store(const std::string& buffer,
+                                  const std::string& moveOriginatorAET,
                                   uint16_t moveOriginatorID)
   {
     if (buffer.size() > 0)
-      Store(reinterpret_cast<const char*>(&buffer[0]), buffer.size(), moveOriginatorID);
+      Store(reinterpret_cast<const char*>(&buffer[0]), buffer.size(), moveOriginatorAET, moveOriginatorID);
     else
-      Store(NULL, 0, moveOriginatorID);
+      Store(NULL, 0, moveOriginatorAET, moveOriginatorID);
   }
 
   void DicomUserConnection::StoreFile(const std::string& path,
+                                      const std::string& moveOriginatorAET,
                                       uint16_t moveOriginatorID)
   {
     // Prepare an input stream for the file
     DcmInputFileStream is(path.c_str());
-    pimpl_->Store(is, *this, moveOriginatorID);
+    pimpl_->Store(is, *this, moveOriginatorAET, moveOriginatorID);
   }
 
   bool DicomUserConnection::Echo()
@@ -1102,14 +1111,16 @@ namespace Orthanc
 
   void DicomUserConnection::SetTimeout(uint32_t seconds)
   {
-    if (seconds <= 0)
+    if (seconds == 0)
     {
-      throw OrthancException(ErrorCode_ParameterOutOfRange);
+      DisableTimeout();
+    }
+    else
+    {
+      dcmConnectionTimeout.set(seconds);
+      pimpl_->dimseTimeout_ = seconds;
+      pimpl_->acseTimeout_ = 10;  // Timeout used during association negociation
     }
-
-    dcmConnectionTimeout.set(seconds);
-    pimpl_->dimseTimeout_ = seconds;
-    pimpl_->acseTimeout_ = 10;
   }
 
 
@@ -1121,7 +1132,7 @@ namespace Orthanc
      */
     dcmConnectionTimeout.set(-1);
     pimpl_->dimseTimeout_ = 0;
-    pimpl_->acseTimeout_ = 10;
+    pimpl_->acseTimeout_ = 10;  // Timeout used during association negociation
   }
 
 
@@ -1191,4 +1202,12 @@ namespace Orthanc
 
     ExecuteFind(result, pimpl_->assoc_, dataset, sopClass, true, NULL, pimpl_->dimseTimeout_);
   }
+
+  
+  void DicomUserConnection::SetDefaultTimeout(uint32_t seconds)
+  {
+    LOG(INFO) << "Default timeout for DICOM connections if Orthanc acts as SCU (client): " 
+              << seconds << " seconds (0 = no timeout)";
+    defaultTimeout_ = seconds;
+  }  
 }
