@@ -1,6 +1,6 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
@@ -33,6 +33,7 @@
 #include "../PrecompiledHeadersServer.h"
 #include "RemoteModalityParameters.h"
 
+#include "../../Core/Logging.h"
 #include "../../Core/OrthancException.h"
 
 #include <boost/lexical_cast.hpp>
@@ -42,7 +43,7 @@ namespace Orthanc
 {
   RemoteModalityParameters::RemoteModalityParameters() :
     aet_("ORTHANC"),
-    host_("localhost"),
+    host_("127.0.0.1"),
     port_(104),
     manufacturer_(ModalityManufacturer_Generic)
   {
@@ -50,7 +51,7 @@ namespace Orthanc
 
   RemoteModalityParameters::RemoteModalityParameters(const std::string& aet,
                                                      const std::string& host,
-                                                     int port,
+                                                     uint16_t port,
                                                      ModalityManufacturer manufacturer)
   {
     SetApplicationEntityTitle(aet);
@@ -60,16 +61,6 @@ namespace Orthanc
   }
 
 
-  void RemoteModalityParameters::SetPort(int port)
-  {
-    if (port <= 0 || port >= 65535)
-    {
-      throw OrthancException(ErrorCode_ParameterOutOfRange);
-    }
-
-    port_ = port;
-  }
-
   void RemoteModalityParameters::FromJson(const Json::Value& modality)
   {
     if (!modality.isArray() ||
@@ -84,13 +75,20 @@ namespace Orthanc
     const Json::Value& portValue = modality.get(2u, "");
     try
     {
-      SetPort(portValue.asInt());
+      int tmp = portValue.asInt();
+
+      if (tmp <= 0 || tmp >= 65535)
+      {
+        throw OrthancException(ErrorCode_ParameterOutOfRange);
+      }
+
+      SetPort(static_cast<uint16_t>(tmp));
     }
     catch (std::runtime_error /* error inside JsonCpp */)
     {
       try
       {
-        SetPort(boost::lexical_cast<int>(portValue.asString()));
+        SetPort(boost::lexical_cast<uint16_t>(portValue.asString()));
       }
       catch (boost::bad_lexical_cast)
       {
@@ -100,7 +98,17 @@ namespace Orthanc
 
     if (modality.size() == 4)
     {
-      SetManufacturer(modality.get(3u, "").asString());
+      const std::string& manufacturer = modality.get(3u, "").asString();
+
+      try
+      {
+        SetManufacturer(manufacturer);
+      }
+      catch (OrthancException&)
+      {
+        LOG(ERROR) << "Unknown modality manufacturer: \"" << manufacturer << "\"";
+        throw;
+      }
     }
     else
     {
