@@ -168,15 +168,8 @@ namespace Orthanc
 
           try
           {
-            const Encoding defaultEncoding = Configuration::GetDefaultEncoding();
-            FromDcmtkBridge::Convert(summary, **imageDataSet, 
-                                     ORTHANC_MAXIMUM_TAG_LENGTH,
-                                     defaultEncoding);
-            FromDcmtkBridge::ToJson(dicomJson, **imageDataSet,
-                                    DicomToJsonFormat_Full, 
-                                    DicomToJsonFlags_Default, 
-                                    ORTHANC_MAXIMUM_TAG_LENGTH,
-                                    defaultEncoding);
+            Configuration::ExtractDicomSummary(summary, **imageDataSet);
+            Configuration::ExtractDicomAsJson(dicomJson, **imageDataSet);
 
             if (!FromDcmtkBridge::SaveToMemoryBuffer(buffer, **imageDataSet))
             {
@@ -219,7 +212,7 @@ namespace Orthanc
 
                 if (e.GetErrorCode() == ErrorCode_InexistentTag)
                 {
-                  Toolbox::LogMissingRequiredTag(summary);
+                  ServerToolbox::LogMissingRequiredTag(summary);
                 }
                 else
                 {
