@@ -107,10 +107,34 @@ namespace Orthanc
   }
 
 
-  static void ExtractTagFromInstances(std::set<std::string>& target,
-                                      ServerContext& context,
-                                      const DicomTag& tag,
-                                      const std::list<std::string>& instances)
+  static bool ExtractMetadata(std::set<std::string>& target,
+                              ServerIndex& index,
+                              MetadataType metadata,
+                              const std::list<std::string>& resources)
+  {
+    for (std::list<std::string>::const_iterator
+           it = resources.begin(); it != resources.end(); ++it)
+    {
+      std::string value;
+      if (index.LookupMetadata(value, *it, metadata))
+      {
+        target.insert(value);
+      }
+      else
+      {
+        // This metadata is unavailable for some resource, give up
+        return false;
+      }
+    }
+
+    return true;
+  }  
+
+
+  static void ExtractTagFromInstancesOnDisk(std::set<std::string>& target,
+                                            ServerContext& context,
+                                            const DicomTag& tag,
+                                            const std::list<std::string>& instances)
   {
     // WARNING: This function is slow, as it reads the JSON file
     // summarizing each instance of interest from the hard drive.
@@ -121,7 +145,7 @@ namespace Orthanc
            it = instances.begin(); it != instances.end(); ++it)
     {
       Json::Value dicom;
-      context.ReadJson(dicom, *it);
+      context.ReadDicomAsJson(dicom, *it);
 
       if (dicom.isMember(formatted))
       {
@@ -226,10 +250,16 @@ namespace Orthanc
 
     if (query.HasTag(DICOM_TAG_SOP_CLASSES_IN_STUDY))
     {
-      if (Configuration::GetGlobalBoolParameter("AllowFindSopClassesInStudy", false))
+      std::set<std::string> values;
+
+      if (ExtractMetadata(values, index, MetadataType_Instance_SopClassUid, instances))
       {
-        std::set<std::string> values;
-        ExtractTagFromInstances(values, context, DICOM_TAG_SOP_CLASS_UID, instances);
+        // The metadata "SopClassUid" is available for each of these instances
+        StoreSetOfStrings(result, DICOM_TAG_SOP_CLASSES_IN_STUDY, values);
+      }
+      else if (Configuration::GetGlobalBoolParameter("AllowFindSopClassesInStudy", false))
+      {
+        ExtractTagFromInstancesOnDisk(values, context, DICOM_TAG_SOP_CLASS_UID, instances);
         StoreSetOfStrings(result, DICOM_TAG_SOP_CLASSES_IN_STUDY, values);
       }
       else
@@ -401,7 +431,7 @@ namespace Orthanc
           for (Json::Value::ArrayIndex i = 0; i < source["Value"].size(); i++)
           {
             Json::Value item;
-            Toolbox::SimplifyTags(item, source["Value"][i], DicomToJsonFormat_Short);
+            ServerToolbox::SimplifyTags(item, source["Value"][i], DicomToJsonFormat_Short);
             content.append(item);
           }
 
@@ -456,62 +486,27 @@ namespace Orthanc
                                                  const std::string& remoteAet,
                                                  const std::string& calledAet)
   {
-    Json::Value output;
+    static const char* LUA_CALLBACK = "IncomingFindRequestFilter";
 
+    LuaScripting::Locker locker(context_.GetLua());
+    if (!locker.GetLua().IsExistingFunction(LUA_CALLBACK))
+    {
+      return false;
+    }
+    else
     {
-      LuaScripting::Locker locker(context_.GetLua());
-      static const char* NAME = "IncomingFindRequestFilter";
-      
-      if (!locker.GetLua().IsExistingFunction(NAME))
-      {
-        return false;
-      }
-
-      Json::Value tmp = Json::objectValue;
-      DicomArray a(source);
-
-      for (size_t i = 0; i < a.GetSize(); i++)
-      {
-        const DicomValue& v = a.GetElement(i).GetValue();
-        std::string s = (v.IsNull() || v.IsBinary()) ? "" : v.GetContent();
-        tmp[a.GetElement(i).GetTag().Format()] = s;
-      }
-
       Json::Value origin = Json::objectValue;
       origin["RemoteIp"] = remoteIp;
       origin["RemoteAet"] = remoteAet;
       origin["CalledAet"] = calledAet;
 
-      LuaFunctionCall call(locker.GetLua(), NAME);
-      call.PushJson(tmp);
+      LuaFunctionCall call(locker.GetLua(), LUA_CALLBACK);
+      call.PushDicom(source);
       call.PushJson(origin);
+      FromDcmtkBridge::ExecuteToDicom(target, call);
 
-      call.ExecuteToJson(output, true);
+      return true;
     }
-
-    // The Lua context is released at this point
-
-    if (output.type() != Json::objectValue)
-    {
-      LOG(ERROR) << "Lua: IncomingFindRequestFilter must return a table";
-      throw OrthancException(ErrorCode_LuaBadOutput);
-    }
-
-    Json::Value::Members members = output.getMemberNames();
-
-    for (size_t i = 0; i < members.size(); i++)
-    {
-      if (output[members[i]].type() != Json::stringValue)
-      {
-        LOG(ERROR) << "Lua: IncomingFindRequestFilter must return a table mapping names of DICOM tags to strings";
-        throw OrthancException(ErrorCode_LuaBadOutput);
-      }
-
-      DicomTag tag(FromDcmtkBridge::ParseTag(members[i]));
-      target.SetValue(tag, output[members[i]].asString(), false);
-    }
-
-    return true;
   }
 
 
@@ -582,7 +577,7 @@ namespace Orthanc
       if (!query.GetElement(i).GetValue().IsNull())
       {
         LOG(INFO) << "  " << query.GetElement(i).GetTag()
-                  << "  " << FromDcmtkBridge::GetName(query.GetElement(i).GetTag())
+                  << "  " << FromDcmtkBridge::GetTagName(query.GetElement(i))
                   << " = " << query.GetElement(i).GetValue().GetContent();
       }
     }
@@ -591,7 +586,7 @@ namespace Orthanc
          it != sequencesToReturn.end(); ++it)
     {
       LOG(INFO) << "  (" << it->Format()
-                << ")  " << FromDcmtkBridge::GetName(*it)
+                << ")  " << FromDcmtkBridge::GetTagName(*it, "")
                 << " : sequence tag whose content will be copied";
     }
 
@@ -604,16 +599,17 @@ namespace Orthanc
 
     for (size_t i = 0; i < query.GetSize(); i++)
     {
-      const DicomTag tag = query.GetElement(i).GetTag();
+      const DicomElement& element = query.GetElement(i);
+      const DicomTag tag = element.GetTag();
 
-      if (query.GetElement(i).GetValue().IsNull() ||
+      if (element.GetValue().IsNull() ||
           tag == DICOM_TAG_QUERY_RETRIEVE_LEVEL ||
           tag == DICOM_TAG_SPECIFIC_CHARACTER_SET)
       {
         continue;
       }
 
-      std::string value = query.GetElement(i).GetValue().GetContent();
+      std::string value = element.GetValue().GetContent();
       if (value.size() == 0)
       {
         // An empty string corresponds to a "*" wildcard constraint, so we ignore it
@@ -637,7 +633,7 @@ namespace Orthanc
       else
       {
         LOG(INFO) << "Because of a patch for the manufacturer of the remote modality, " 
-                  << "ignoring constraint on tag (" << tag.Format() << ") " << FromDcmtkBridge::GetName(tag);
+                  << "ignoring constraint on tag (" << tag.Format() << ") " << FromDcmtkBridge::GetTagName(element);
       }
     }
 
@@ -651,13 +647,15 @@ namespace Orthanc
     std::vector<std::string> resources, instances;
     context_.GetIndex().FindCandidates(resources, instances, finder);
 
+    LOG(INFO) << "Number of candidate resources after fast DB filtering: " << resources.size();
+
     assert(resources.size() == instances.size());
     bool complete = true;
 
     for (size_t i = 0; i < instances.size(); i++)
     {
       Json::Value dicom;
-      context_.ReadJson(dicom, instances[i]);
+      context_.ReadDicomAsJson(dicom, instances[i]);
       
       if (finder.IsMatch(dicom))
       {
@@ -669,7 +667,7 @@ namespace Orthanc
         }
         else
         {
-          std::auto_ptr<DicomMap> counters(ComputeCounters(context_, instances[i], level, input));
+          std::auto_ptr<DicomMap> counters(ComputeCounters(context_, instances[i], level, *filteredInput));
           AddAnswer(answers, dicom, query, sequencesToReturn, counters.get());
         }
       }
