@@ -47,26 +47,16 @@
 #include <boost/lexical_cast.hpp>
 #include <boost/filesystem.hpp>
 #include <curl/curl.h>
-#include <boost/thread.hpp>
+#include <boost/thread/recursive_mutex.hpp>
 
 
-#if ORTHANC_SSL_ENABLED == 1
-// For OpenSSL initialization and finalization
-#include <openssl/conf.h>
-#include <openssl/engine.h>
-#include <openssl/err.h>
-#include <openssl/evp.h>
-#include <openssl/ssl.h>
+#if ORTHANC_ENABLE_JPEG == 1
+#  include <dcmtk/dcmjpeg/djdecode.h>
 #endif
 
 
-#if ORTHANC_JPEG_ENABLED == 1
-#include <dcmtk/dcmjpeg/djdecode.h>
-#endif
-
-
-#if ORTHANC_JPEG_LOSSLESS_ENABLED == 1
-#include <dcmtk/dcmjpls/djdecode.h>
+#if ORTHANC_ENABLE_JPEG_LOSSLESS == 1
+#  include <dcmtk/dcmjpls/djdecode.h>
 #endif
 
 
@@ -138,7 +128,7 @@ namespace Orthanc
 
     {
       std::string content;
-      Toolbox::ReadFile(content, path.string());
+      SystemToolbox::ReadFile(content, path.string());
 
       Json::Value tmp;
       Json::Reader reader;
@@ -404,11 +394,12 @@ namespace Orthanc
       const Json::Value& content = configuration["Dictionary"][tags[i]];
       if (content.type() != Json::arrayValue ||
           content.size() < 2 ||
-          content.size() > 4 ||
+          content.size() > 5 ||
           content[0].type() != Json::stringValue ||
           content[1].type() != Json::stringValue ||
           (content.size() >= 3 && content[2].type() != Json::intValue) ||
-          (content.size() >= 4 && content[3].type() != Json::intValue))
+          (content.size() >= 4 && content[3].type() != Json::intValue) ||
+          (content.size() >= 5 && content[4].type() != Json::stringValue))
       {
         throw OrthancException(ErrorCode_BadFileFormat);
       }
@@ -418,8 +409,9 @@ namespace Orthanc
       std::string name = content[1].asString();
       unsigned int minMultiplicity = (content.size() >= 2) ? content[2].asUInt() : 1;
       unsigned int maxMultiplicity = (content.size() >= 3) ? content[3].asUInt() : 1;
+      std::string privateCreator = (content.size() >= 4) ? content[4].asString() : "";
 
-      FromDcmtkBridge::RegisterDictionaryTag(tag, vr, name, minMultiplicity, maxMultiplicity);
+      FromDcmtkBridge::RegisterDictionaryTag(tag, vr, name, minMultiplicity, maxMultiplicity, privateCreator);
     }
   }
 
@@ -471,13 +463,7 @@ namespace Orthanc
   {
     boost::recursive_mutex::scoped_lock lock(globalMutex_);
 
-#if ORTHANC_SSL_ENABLED == 1
-    // https://wiki.openssl.org/index.php/Library_Initialization
-    SSL_library_init();
-    SSL_load_error_strings();
-    OpenSSL_add_all_algorithms();
-    ERR_load_crypto_strings();
-#endif
+    HttpClient::InitializeOpenSsl();
 
     InitializeServerEnumerations();
 
@@ -495,15 +481,15 @@ namespace Orthanc
     RegisterUserMetadata();
     RegisterUserContentType();
 
-    FromDcmtkBridge::InitializeDictionary();
+    FromDcmtkBridge::InitializeDictionary(GetGlobalBoolParameterInternal("LoadPrivateDictionary", true));
     LoadCustomDictionary(configuration_);
 
-#if ORTHANC_JPEG_LOSSLESS_ENABLED == 1
+#if ORTHANC_ENABLE_JPEG_LOSSLESS == 1
     LOG(WARNING) << "Registering JPEG Lossless codecs";
     DJLSDecoderRegistration::registerCodecs();    
 #endif
 
-#if ORTHANC_JPEG_ENABLED == 1
+#if ORTHANC_ENABLE_JPEG == 1
     LOG(WARNING) << "Registering JPEG codecs";
     DJDecoderRegistration::registerCodecs(); 
 #endif
@@ -521,27 +507,17 @@ namespace Orthanc
     boost::recursive_mutex::scoped_lock lock(globalMutex_);
     HttpClient::GlobalFinalize();
 
-#if ORTHANC_JPEG_LOSSLESS_ENABLED == 1
+#if ORTHANC_ENABLE_JPEG_LOSSLESS == 1
     // Unregister JPEG-LS codecs
     DJLSDecoderRegistration::cleanup();
 #endif
 
-#if ORTHANC_JPEG_ENABLED == 1
+#if ORTHANC_ENABLE_JPEG == 1
     // Unregister JPEG codecs
     DJDecoderRegistration::cleanup();
 #endif
 
-#if ORTHANC_SSL_ENABLED == 1
-    // Finalize OpenSSL
-    // https://wiki.openssl.org/index.php/Library_Initialization#Cleanup
-    FIPS_mode_set(0);
-    ENGINE_cleanup();
-    CONF_modules_unload(1);
-    EVP_cleanup();
-    CRYPTO_cleanup_all_ex_data();
-    ERR_remove_state(0);
-    ERR_free_strings();
-#endif
+    HttpClient::FinalizeOpenSsl();
   }
 
 
@@ -577,6 +553,23 @@ namespace Orthanc
   }
 
 
+  unsigned int Configuration::GetGlobalUnsignedIntegerParameter(const std::string& parameter,
+                                                                unsigned int defaultValue)
+  {
+    int v = GetGlobalIntegerParameter(parameter, defaultValue);
+
+    if (v < 0)
+    {
+      LOG(ERROR) << "The configuration option \"" << parameter << "\" must be a positive integer";
+      throw OrthancException(ErrorCode_ParameterOutOfRange);
+    }
+    else
+    {
+      return static_cast<unsigned int>(v);
+    }
+  }
+
+
   bool Configuration::GetGlobalBoolParameter(const std::string& parameter,
                                              bool defaultValue)
   {
@@ -1118,6 +1111,19 @@ namespace Orthanc
   }
 
 
+  void Configuration::SetDefaultEncoding(Encoding encoding)
+  {
+    std::string name = EnumerationToString(encoding);
+
+    {
+      boost::recursive_mutex::scoped_lock lock(globalMutex_);
+      configuration_["DefaultEncoding"] = name;
+    }
+
+    LOG(INFO) << "Default encoding was changed to: " << name;
+  }
+
+
   bool Configuration::HasConfigurationChanged()
   {
     Json::Value starting;
@@ -1132,4 +1138,21 @@ namespace Orthanc
 
     return a != b;
   }
+
+
+  void Configuration::ExtractDicomSummary(DicomMap& target, 
+                                          DcmItem& dataset)
+  {
+    FromDcmtkBridge::ExtractDicomSummary(target, dataset, 
+                                         ORTHANC_MAXIMUM_TAG_LENGTH, GetDefaultEncoding());
+  }
+
+  
+  void Configuration::ExtractDicomAsJson(Json::Value& target, 
+                                         DcmDataset& dataset)
+  {
+    FromDcmtkBridge::ExtractDicomAsJson(target, dataset, 
+                                        DicomToJsonFormat_Full, DicomToJsonFlags_Default, 
+                                        ORTHANC_MAXIMUM_TAG_LENGTH, GetDefaultEncoding());
+  }
 }
