@@ -34,7 +34,6 @@
 #include "OrthancRestApi.h"
 
 #include "../../Core/Logging.h"
-#include "../../Core/Uuid.h"
 #include "../FromDcmtkBridge.h"
 #include "../ServerContext.h"
 #include "../OrthancInitialization.h"
@@ -137,6 +136,16 @@ namespace Orthanc
         ParseReplacements(target, request["Replace"]);
       }
 
+      // The "Keep" operation only makes sense for the tags
+      // StudyInstanceUID, SeriesInstanceUID and SOPInstanceUID. Avoid
+      // this feature as much as possible, as this breaks the DICOM
+      // model of the real world, except if you know exactly what
+      // you're doing!
+      if (request.isMember("Keep"))
+      {
+        ParseListOfTags(target, request["Keep"], TagOperation_Keep);
+      }
+
       return true;
     }
     else
@@ -674,7 +683,7 @@ namespace Orthanc
           throw OrthancException(ErrorCode_InternalError);
         }
 
-        context.ReadJson(siblingTags, siblingInstances.front());
+        context.ReadDicomAsJson(siblingTags, siblingInstances.front());
       }
 
 
@@ -758,7 +767,7 @@ namespace Orthanc
     
     // Inject time-related information
     std::string date, time;
-    Toolbox::GetNowDicom(date, time);
+    SystemToolbox::GetNowDicom(date, time);
     dicom.ReplacePlainString(DICOM_TAG_ACQUISITION_DATE, date);
     dicom.ReplacePlainString(DICOM_TAG_ACQUISITION_TIME, time);
     dicom.ReplacePlainString(DICOM_TAG_CONTENT_DATE, date);
