@@ -38,7 +38,7 @@
 #include "../../Core/Compression/HierarchicalZipWriter.h"
 #include "../../Core/HttpServer/FilesystemHttpSender.h"
 #include "../../Core/Logging.h"
-#include "../../Core/Uuid.h"
+#include "../../Core/TemporaryFile.h"
 #include "../ServerContext.h"
 
 #include <stdio.h>
@@ -524,7 +524,7 @@ namespace Orthanc
                                const FileInfo& dicom)
       {
         std::string content;
-        context_.ReadFile(content, dicom);
+        context_.ReadAttachment(content, dicom);
 
         char filename[24];
         snprintf(filename, sizeof(filename) - 1, instanceFormat_, countInstances_);
@@ -547,7 +547,7 @@ namespace Orthanc
         const bool isZip64 = IsZip64Required(stats.GetUncompressedSize(), stats.GetInstancesCount());
 
         // Create a RAII for the temporary file to manage the ZIP file
-        Toolbox::TemporaryFile tmp;
+        TemporaryFile tmp;
 
         {
           // Create a ZIP writer
@@ -612,7 +612,7 @@ namespace Orthanc
         writer_.OpenFile(filename.c_str());
 
         std::string content;
-        context_.ReadFile(content, dicom);
+        context_.ReadAttachment(content, dicom);
         writer_.Write(content);
 
         ParsedDicomFile parsed(content);
@@ -634,7 +634,7 @@ namespace Orthanc
         const bool isZip64 = IsZip64Required(stats.GetUncompressedSize(), stats.GetInstancesCount());
 
         // Create a RAII for the temporary file to manage the ZIP file
-        Toolbox::TemporaryFile tmp;
+        TemporaryFile tmp;
 
         {
           // Create a ZIP writer
