@@ -33,13 +33,15 @@
 #include "../PrecompiledHeadersServer.h"
 #include "OrthancRestApi.h"
 
-#include "../../Core/Logging.h"
 #include "../../Core/HttpServer/HttpContentNegociation.h"
-#include "../ServerToolbox.h"
+#include "../../Core/Logging.h"
 #include "../FromDcmtkBridge.h"
-#include "../ServerContext.h"
-#include "../SliceOrdering.h"
 #include "../Internals/DicomImageDecoder.h"
+#include "../OrthancInitialization.h"
+#include "../Search/LookupResource.h"
+#include "../ServerContext.h"
+#include "../ServerToolbox.h"
+#include "../SliceOrdering.h"
 
 
 namespace Orthanc
@@ -186,11 +188,11 @@ namespace Orthanc
     std::string publicId = call.GetUriComponent("id", "");
 
     std::string dicom;
-    context.ReadFile(dicom, publicId, FileContentType_Dicom);
+    context.ReadDicom(dicom, publicId);
 
     std::string target;
     call.BodyToString(target);
-    Toolbox::WriteFile(dicom, target);
+    SystemToolbox::WriteFile(dicom, target);
 
     call.GetOutput().AnswerBuffer("{}", "application/json");
   }
@@ -206,15 +208,17 @@ namespace Orthanc
     if (simplify)
     {
       Json::Value full;
-      context.ReadJson(full, publicId);
+      context.ReadDicomAsJson(full, publicId);
 
       Json::Value simplified;
-      Toolbox::SimplifyTags(simplified, full, DicomToJsonFormat_Human);
+      ServerToolbox::SimplifyTags(simplified, full, DicomToJsonFormat_Human);
       call.GetOutput().AnswerJson(simplified);
     }
     else
     {
-      context.AnswerAttachment(call.GetOutput(), publicId, FileContentType_DicomAsJson);
+      std::string full;
+      context.ReadDicomAsJson(full, publicId);
+      call.GetOutput().AnswerBuffer(full, "application/json");
     }
   }
 
@@ -374,12 +378,12 @@ namespace Orthanc
     {
       std::string publicId = call.GetUriComponent("id", "");
 
-#if ORTHANC_PLUGINS_ENABLED == 1
+#if ORTHANC_ENABLE_PLUGINS == 1
       if (context.GetPlugins().HasCustomImageDecoder())
       {
         // TODO create a cache of file
         std::string dicomContent;
-        context.ReadFile(dicomContent, publicId, FileContentType_Dicom);
+        context.ReadDicom(dicomContent, publicId);
         decoded.reset(context.GetPlugins().DecodeUnsafe(dicomContent.c_str(), dicomContent.size(), frame));
 
         /**
@@ -449,9 +453,9 @@ namespace Orthanc
 
     std::string publicId = call.GetUriComponent("id", "");
     std::string dicomContent;
-    context.ReadFile(dicomContent, publicId, FileContentType_Dicom);
+    context.ReadDicom(dicomContent, publicId);
 
-#if ORTHANC_PLUGINS_ENABLED == 1
+#if ORTHANC_ENABLE_PLUGINS == 1
     IDicomImageDecoder& decoder = context.GetPlugins();
 #else
     DefaultDicomImageDecoder decoder;  // This is Orthanc's built-in decoder
@@ -562,6 +566,10 @@ namespace Orthanc
       OrthancRestApi::GetIndex(call).DeleteMetadata(publicId, metadata);
       call.GetOutput().AnswerBuffer("", "text/plain");
     }
+    else
+    {
+      call.GetOutput().SignalError(HttpStatus_403_Forbidden);
+    }
   }
 
 
@@ -582,6 +590,10 @@ namespace Orthanc
       OrthancRestApi::GetIndex(call).SetMetadata(publicId, metadata, value);
       call.GetOutput().AnswerBuffer("", "text/plain");
     }
+    else
+    {
+      call.GetOutput().SignalError(HttpStatus_403_Forbidden);
+    }
   }
 
 
@@ -676,7 +688,7 @@ namespace Orthanc
     {
       // Return the raw data (possibly compressed), as stored on the filesystem
       std::string content;
-      context.ReadFile(content, publicId, type, false);
+      context.ReadAttachment(content, publicId, type, false);
       call.GetOutput().AnswerBuffer(content, "application/octet-stream");
     }
   }
@@ -745,7 +757,7 @@ namespace Orthanc
 
     // First check whether the compressed data is correctly stored in the disk
     std::string data;
-    context.ReadFile(data, publicId, StringToContentType(name), false);
+    context.ReadAttachment(data, publicId, StringToContentType(name), false);
 
     std::string actualMD5;
     Toolbox::ComputeMD5(actualMD5, data);
@@ -760,7 +772,7 @@ namespace Orthanc
       }
       else
       {
-        context.ReadFile(data, publicId, StringToContentType(name), true);        
+        context.ReadAttachment(data, publicId, StringToContentType(name), true);        
         Toolbox::ComputeMD5(actualMD5, data);
         ok = (actualMD5 == info.GetUncompressedMD5());
       }
@@ -792,6 +804,10 @@ namespace Orthanc
     {
       call.GetOutput().AnswerBuffer("{}", "application/json");
     }
+    else
+    {
+      call.GetOutput().SignalError(HttpStatus_403_Forbidden);
+    }
   }
 
 
@@ -803,11 +819,33 @@ namespace Orthanc
     std::string name = call.GetUriComponent("name", "");
     FileContentType contentType = StringToContentType(name);
 
-    if (IsUserContentType(contentType))  // It is forbidden to delete internal attachments
+    bool allowed;
+    if (IsUserContentType(contentType))
+    {
+      allowed = true;
+    }
+    else if (Configuration::GetGlobalBoolParameter("StoreDicom", true) &&
+             contentType == FileContentType_DicomAsJson)
+    {
+      allowed = true;
+    }
+    else
+    {
+      // It is forbidden to delete internal attachments, except for
+      // the "DICOM as JSON" summary as of Orthanc 1.2.0 (this summary
+      // would be automatically reconstructed on the next GET call)
+      allowed = false;
+    }
+
+    if (allowed) 
     {
       OrthancRestApi::GetIndex(call).DeleteAttachment(publicId, contentType);
       call.GetOutput().AnswerBuffer("{}", "application/json");
     }
+    else
+    {
+      call.GetOutput().SignalError(HttpStatus_403_Forbidden);
+    }
   }
 
 
@@ -870,7 +908,7 @@ namespace Orthanc
 
       try
       {
-        context.ReadJson(tags, *it);
+        context.ReadDicomAsJson(tags, *it);
       }
       catch (OrthancException&)
       {
@@ -937,7 +975,7 @@ namespace Orthanc
       if (simplify)
       {
         Json::Value simplified;
-        Toolbox::SimplifyTags(simplified, sharedTags, DicomToJsonFormat_Human);
+        ServerToolbox::SimplifyTags(simplified, sharedTags, DicomToJsonFormat_Human);
         call.GetOutput().AnswerJson(simplified);
       }
       else
@@ -988,7 +1026,7 @@ namespace Orthanc
       publicId = instances.front();
     }
 
-    context.ReadJson(tags, publicId);
+    context.ReadDicomAsJson(tags, publicId);
     
     // Filter the tags of the instance according to the module
     Json::Value result = Json::objectValue;
@@ -1004,7 +1042,7 @@ namespace Orthanc
     if (simplify)
     {
       Json::Value simplified;
-      Toolbox::SimplifyTags(simplified, result, DicomToJsonFormat_Human);
+      ServerToolbox::SimplifyTags(simplified, result, DicomToJsonFormat_Human);
       call.GetOutput().AnswerJson(simplified);
     }
     else
@@ -1205,12 +1243,12 @@ namespace Orthanc
          it != instances.end(); ++it)
     {
       Json::Value full;
-      context.ReadJson(full, *it);
+      context.ReadDicomAsJson(full, *it);
 
       if (simplify)
       {
         Json::Value simplified;
-        Toolbox::SimplifyTags(simplified, full, DicomToJsonFormat_Human);
+        ServerToolbox::SimplifyTags(simplified, full, DicomToJsonFormat_Human);
         result[*it] = simplified;
       }
       else
@@ -1294,7 +1332,7 @@ namespace Orthanc
     bool simplify = call.HasArgument("simplify");
 
     std::string dicomContent;
-    context.ReadFile(dicomContent, publicId, FileContentType_Dicom);
+    context.ReadDicom(dicomContent, publicId);
 
     // TODO Consider using "DicomMap::ParseDicomMetaInformation()" to
     // speed up things here
@@ -1307,7 +1345,7 @@ namespace Orthanc
     if (simplify)
     {
       Json::Value simplified;
-      Toolbox::SimplifyTags(simplified, header, DicomToJsonFormat_Human);
+      ServerToolbox::SimplifyTags(simplified, header, DicomToJsonFormat_Human);
       call.GetOutput().AnswerJson(simplified);
     }
     else
@@ -1317,6 +1355,41 @@ namespace Orthanc
   }
 
 
+  static void InvalidateTags(RestApiPostCall& call)
+  {
+    ServerIndex& index = OrthancRestApi::GetIndex(call);
+    
+    // Loop over the instances, grouping them by parent studies so as
+    // to avoid large memory consumption
+    std::list<std::string> studies;
+    index.GetAllUuids(studies, ResourceType_Study);
+
+    for (std::list<std::string>::const_iterator 
+           study = studies.begin(); study != studies.end(); ++study)
+    {
+      std::list<std::string> instances;
+      index.GetChildInstances(instances, *study);
+
+      for (std::list<std::string>::const_iterator 
+             instance = instances.begin(); instance != instances.end(); ++instance)
+      {
+        index.DeleteAttachment(*instance, FileContentType_DicomAsJson);
+      }
+    }
+
+    call.GetOutput().AnswerBuffer("", "text/plain");
+  }
+
+
+  template <enum ResourceType type>
+  static void ReconstructResource(RestApiPostCall& call)
+  {
+    ServerContext& context = OrthancRestApi::GetContext(call);
+    ServerToolbox::ReconstructResource(context, call.GetUriComponent("id", ""));
+    call.GetOutput().AnswerBuffer("", "text/plain");
+  }
+
+
   void OrthancRestApi::RegisterResources()
   {
     Register("/instances", ListResources<ResourceType_Instance>);
@@ -1391,6 +1464,7 @@ namespace Orthanc
     Register("/{resourceType}/{id}/attachments/{name}/uncompress", ChangeAttachmentCompression<CompressionType_None>);
     Register("/{resourceType}/{id}/attachments/{name}/verify-md5", VerifyAttachment);
 
+    Register("/tools/invalidate-tags", InvalidateTags);
     Register("/tools/lookup", Lookup);
     Register("/tools/find", Find);
 
@@ -1415,5 +1489,10 @@ namespace Orthanc
     Register("/instances/{id}/content/*", GetRawContent);
 
     Register("/series/{id}/ordered-slices", OrderSlices);
+
+    Register("/patients/{id}/reconstruct", ReconstructResource<ResourceType_Patient>);
+    Register("/studies/{id}/reconstruct", ReconstructResource<ResourceType_Study>);
+    Register("/series/{id}/reconstruct", ReconstructResource<ResourceType_Series>);
+    Register("/instances/{id}/reconstruct", ReconstructResource<ResourceType_Instance>);
   }
 }
