@@ -34,10 +34,43 @@
 #include "QueryRetrieveHandler.h"
 
 #include "OrthancInitialization.h"
+#include "FromDcmtkBridge.h"
 
 
 namespace Orthanc
 {
+  static void FixQuery(DicomMap& query,
+                       ServerContext& context,
+                       const std::string& modality)
+  {
+    static const char* LUA_CALLBACK = "OutgoingFindRequestFilter";
+
+    LuaScripting::Locker locker(context.GetLua());
+    if (locker.GetLua().IsExistingFunction(LUA_CALLBACK))
+    {
+      LuaFunctionCall call(locker.GetLua(), LUA_CALLBACK);
+      call.PushDicom(query);
+      call.PushJson(modality);
+      FromDcmtkBridge::ExecuteToDicom(query, call);
+    }
+  }
+
+
+  static void FixQuery(DicomMap& query,
+                       ModalityManufacturer manufacturer)
+  {
+    /**
+     * Introduce patches for specific manufacturers below.
+     **/
+
+    switch (manufacturer)
+    {
+      default:
+        break;
+    }
+  }
+
+
   void QueryRetrieveHandler::Invalidate()
   {
     done_ = false;
@@ -49,8 +82,20 @@ namespace Orthanc
   {
     if (!done_)
     {
-      ReusableDicomUserConnection::Locker locker(context_.GetReusableDicomUserConnection(), localAet_, modality_);
-      locker.GetConnection().Find(answers_, level_, query_);
+      // Firstly, fix the content of the query for specific manufacturers
+      DicomMap fixed;
+      fixed.Assign(query_);
+      FixQuery(fixed, modality_.GetManufacturer());
+
+      // Secondly, possibly fix the query with the user-provider Lua callback
+      FixQuery(fixed, context_, modality_.GetApplicationEntityTitle()); 
+
+      {
+        // Finally, run the C-FIND SCU against the fixed query
+        ReusableDicomUserConnection::Locker locker(context_.GetReusableDicomUserConnection(), localAet_, modality_);
+        locker.GetConnection().Find(answers_, level_, fixed);
+      }
+
       done_ = true;
     }
   }
@@ -60,7 +105,8 @@ namespace Orthanc
     context_(context),
     localAet_(context.GetDefaultLocalApplicationEntityTitle()),
     done_(false),
-    level_(ResourceType_Study)
+    level_(ResourceType_Study),
+    answers_(false)
   {
   }
 
@@ -99,7 +145,7 @@ namespace Orthanc
                                        size_t i)
   {
     Run();
-    answers_.GetAnswer(i).Convert(target);
+    answers_.GetAnswer(i).ExtractDicomSummary(target);
   }
 
 
