@@ -52,6 +52,7 @@
 #include "Scheduler/StorePeerCommand.h"
 #include "OrthancRestApi/OrthancRestApi.h"
 #include "../Plugins/Engine/OrthancPlugins.h"
+#include "Search/LookupResource.h"
 
 
 #define ENABLE_DICOM_CACHE  1
@@ -85,12 +86,24 @@ namespace Orthanc
         {
           try
           {
-            it->GetListener().SignalChange(change);
+            try
+            {
+              it->GetListener().SignalChange(change);
+            }
+            catch (std::bad_alloc&)
+            {
+              LOG(ERROR) << "Not enough memory while signaling a change";
+            }
+            catch (...)
+            {
+              throw OrthancException(ErrorCode_InternalError);
+            }
           }
           catch (OrthancException& e)
           {
             LOG(ERROR) << "Error in the " << it->GetDescription() 
-                       << " callback while signaling a change: " << e.What();
+                       << " callback while signaling a change: " << e.What()
+                       << " (code " << e.GetErrorCode() << ")";
           }
         }
       }
@@ -106,16 +119,16 @@ namespace Orthanc
     storeMD5_(true),
     provider_(*this),
     dicomCache_(provider_, DICOM_CACHE_SIZE),
-    scheduler_(Configuration::GetGlobalIntegerParameter("LimitJobs", 10)),
+    scheduler_(Configuration::GetGlobalUnsignedIntegerParameter("LimitJobs", 10)),
     lua_(*this),
-#if ORTHANC_PLUGINS_ENABLED == 1
+#if ORTHANC_ENABLE_PLUGINS == 1
     plugins_(NULL),
 #endif
     done_(false),
-    queryRetrieveArchive_(Configuration::GetGlobalIntegerParameter("QueryRetrieveSize", 10)),
+    queryRetrieveArchive_(Configuration::GetGlobalUnsignedIntegerParameter("QueryRetrieveSize", 10)),
     defaultLocalAet_(Configuration::GetGlobalStringParameter("DicomAet", "ORTHANC"))
   {
-    uint64_t s = Configuration::GetGlobalIntegerParameter("DicomAssociationCloseDelay", 5);  // In seconds
+    uint64_t s = Configuration::GetGlobalUnsignedIntegerParameter("DicomAssociationCloseDelay", 5);  // In seconds
     scu_.SetMillisecondsBeforeClose(s * 1000);  // Milliseconds are expected here
 
     listeners_.push_back(ServerListener(lua_, "Lua"));
@@ -189,7 +202,7 @@ namespace Orthanc
       resultPublicId = hasher.HashInstance();
 
       Json::Value simplifiedTags;
-      Toolbox::SimplifyTags(simplifiedTags, dicom.GetJson(), DicomToJsonFormat_Human);
+      ServerToolbox::SimplifyTags(simplifiedTags, dicom.GetJson(), DicomToJsonFormat_Human);
 
       // Test if the instance must be filtered out
       bool accepted = true;
@@ -210,7 +223,8 @@ namespace Orthanc
           catch (OrthancException& e)
           {
             LOG(ERROR) << "Error in the " << it->GetDescription() 
-                       << " callback while receiving an instance: " << e.What();
+                       << " callback while receiving an instance: " << e.What()
+                       << " (code " << e.GetErrorCode() << ")";
             throw;
           }
         }
@@ -287,7 +301,8 @@ namespace Orthanc
           catch (OrthancException& e)
           {
             LOG(ERROR) << "Error in the " << it->GetDescription() 
-                       << " callback while receiving an instance: " << e.What();
+                       << " callback while receiving an instance: " << e.What()
+                       << " (code " << e.GetErrorCode() << ")";
           }
         }
       }
@@ -298,7 +313,7 @@ namespace Orthanc
     {
       if (e.GetErrorCode() == ErrorCode_InexistentTag)
       {
-        Toolbox::LogMissingRequiredTag(dicom.GetSummary());
+        ServerToolbox::LogMissingRequiredTag(dicom.GetSummary());
       }
 
       throw;
@@ -367,35 +382,67 @@ namespace Orthanc
   }
 
 
-  void ServerContext::ReadJson(Json::Value& result,
-                               const std::string& instancePublicId)
+  void ServerContext::ReadDicomAsJson(std::string& result,
+                                      const std::string& instancePublicId)
   {
-    std::string s;
-    ReadFile(s, instancePublicId, FileContentType_DicomAsJson);
+    FileInfo attachment;
+    if (index_.LookupAttachment(attachment, instancePublicId, FileContentType_DicomAsJson))
+    {
+      ReadAttachment(result, attachment);
+      return;
+    }
+
+    // The "DICOM as JSON" summary is not available from the Orthanc
+    // store (most probably deleted), reconstruct it from the DICOM file
+    std::string dicom;
+    ReadDicom(dicom, instancePublicId);
+
+    LOG(INFO) << "Reconstructing the missing DICOM-as-JSON summary for instance: " << instancePublicId;
+    
+    ParsedDicomFile parsed(dicom);
+
+    Json::Value summary;
+    parsed.DatasetToJson(summary);
+
+    result = summary.toStyledString();
+
+    if (!AddAttachment(instancePublicId, FileContentType_DicomAsJson, result.c_str(), result.size()))
+    {
+      LOG(WARNING) << "Cannot associate the DICOM-as-JSON summary to instance: " << instancePublicId;
+      throw OrthancException(ErrorCode_InternalError);
+    }
+  }
+
+
+  void ServerContext::ReadDicomAsJson(Json::Value& result,
+                                      const std::string& instancePublicId)
+  {
+    std::string tmp;
+    ReadDicomAsJson(tmp, instancePublicId);
 
     Json::Reader reader;
-    if (!reader.parse(s, result))
+    if (!reader.parse(tmp, result))
     {
       throw OrthancException(ErrorCode_CorruptedFile);
     }
   }
 
 
-  void ServerContext::ReadFile(std::string& result,
-                               const std::string& instancePublicId,
-                               FileContentType content,
-                               bool uncompressIfNeeded)
+  void ServerContext::ReadAttachment(std::string& result,
+                                     const std::string& instancePublicId,
+                                     FileContentType content,
+                                     bool uncompressIfNeeded)
   {
     FileInfo attachment;
     if (!index_.LookupAttachment(attachment, instancePublicId, content))
     {
+      LOG(WARNING) << "Unable to read attachment " << EnumerationToString(content) << " of instance " << instancePublicId;
       throw OrthancException(ErrorCode_InternalError);
     }
 
     if (uncompressIfNeeded)
     {
-      StorageAccessor accessor(area_);
-      accessor.Read(result, attachment);
+      ReadAttachment(result, attachment);
     }
     else
     {
@@ -406,18 +453,19 @@ namespace Orthanc
   }
 
 
-  void ServerContext::ReadFile(std::string& result,
-                               const FileInfo& file)
+  void ServerContext::ReadAttachment(std::string& result,
+                                     const FileInfo& attachment)
   {
+    // This will decompress the attachment
     StorageAccessor accessor(area_);
-    accessor.Read(result, file);
+    accessor.Read(result, attachment);
   }
 
 
   IDynamicObject* ServerContext::DicomCacheProvider::Provide(const std::string& instancePublicId)
   {
     std::string content;
-    context_.ReadFile(content, instancePublicId, FileContentType_Dicom);
+    context_.ReadDicom(content, instancePublicId);
     return new ParsedDicomFile(content);
   }
 
@@ -489,7 +537,7 @@ namespace Orthanc
   }
 
 
-#if ORTHANC_PLUGINS_ENABLED == 1
+#if ORTHANC_ENABLE_PLUGINS == 1
   void ServerContext::SetPlugins(OrthancPlugins& plugins)
   {
     boost::recursive_mutex::scoped_lock lock(listenersMutex_);
@@ -544,7 +592,7 @@ namespace Orthanc
 
   bool ServerContext::HasPlugins() const
   {
-#if ORTHANC_PLUGINS_ENABLED == 1
+#if ORTHANC_ENABLE_PLUGINS == 1
     return (plugins_ != NULL);
 #else
     return false;
@@ -566,7 +614,7 @@ namespace Orthanc
     for (size_t i = 0; i < instances.size(); i++)
     {
       Json::Value dicom;
-      ReadJson(dicom, instances[i]);
+      ReadDicomAsJson(dicom, instances[i]);
       
       if (lookup.IsMatch(dicom))
       {
