@@ -47,7 +47,6 @@
 #include "Scheduler/ServerScheduler.h"
 #include "ServerIndex.h"
 #include "OrthancHttpHandler.h"
-#include "Search/LookupResource.h"
 
 #include <boost/filesystem.hpp>
 #include <boost/thread.hpp>
@@ -121,7 +120,7 @@ namespace Orthanc
 
     LuaScripting lua_;
 
-#if ORTHANC_PLUGINS_ENABLED == 1
+#if ORTHANC_ENABLE_PLUGINS == 1
     OrthancPlugins* plugins_;
 #endif
 
@@ -192,17 +191,26 @@ namespace Orthanc
                                      FileContentType attachmentType,
                                      CompressionType compression);
 
-    void ReadJson(Json::Value& result,
-                  const std::string& instancePublicId);
+    void ReadDicomAsJson(std::string& result,
+                         const std::string& instancePublicId);
 
+    void ReadDicomAsJson(Json::Value& result,
+                         const std::string& instancePublicId);
+
+    void ReadDicom(std::string& dicom,
+                   const std::string& instancePublicId)
+    {
+      ReadAttachment(dicom, instancePublicId, FileContentType_Dicom, true);
+    }
+    
     // TODO CACHING MECHANISM AT THIS POINT
-    void ReadFile(std::string& result,
-                  const std::string& instancePublicId,
-                  FileContentType content,
-                  bool uncompressIfNeeded = true);
-
-    void ReadFile(std::string& result,
-                  const FileInfo& file);
+    void ReadAttachment(std::string& result,
+                        const std::string& instancePublicId,
+                        FileContentType content,
+                        bool uncompressIfNeeded);
+    
+    void ReadAttachment(std::string& result,
+                        const FileInfo& attachment);
 
     void SetStoreMD5ForAttachments(bool storeMD5);
 
@@ -258,7 +266,7 @@ namespace Orthanc
      * Management of the plugins
      **/
 
-#if ORTHANC_PLUGINS_ENABLED == 1
+#if ORTHANC_ENABLE_PLUGINS == 1
     void SetPlugins(OrthancPlugins& plugins);
 
     void ResetPlugins();
