@@ -47,6 +47,7 @@ namespace Orthanc
   class LookupResource;
   class ServerContext;
   class DicomInstanceToStore;
+  class ParsedDicomFile;
 
   class ServerIndex : public boost::noncopyable
   {
@@ -115,6 +116,11 @@ namespace Orthanc
     int64_t CreateResource(const std::string& publicId,
                            ResourceType type);
 
+    void SetInstanceMetadata(std::map<MetadataType, std::string>& instanceMetadata,
+                             int64_t instance,
+                             MetadataType metadata,
+                             const std::string& value);
+
   public:
     ServerIndex(ServerContext& context,
                 IDatabaseWrapper& database);
@@ -268,5 +274,7 @@ namespace Orthanc
     bool LookupParent(std::string& target,
                       const std::string& publicId,
                       ResourceType parentType);
+
+    void ReconstructInstance(ParsedDicomFile& dicom);
   };
 }
