@@ -34,7 +34,7 @@
 
 #include "ServerEnumerations.h"
 #include "../Core/IDynamicObject.h"
-#include "../Core/Toolbox.h"
+#include "../Core/SystemToolbox.h"
 
 #include <string>
 #include <json/value.h>
@@ -58,7 +58,7 @@ namespace Orthanc
       changeType_(changeType),
       resourceType_(resourceType),
       publicId_(publicId),
-      date_(Toolbox::GetNowIsoString())
+      date_(SystemToolbox::GetNowIsoString())
     {
     }
 
