@@ -34,7 +34,12 @@
 
 #include "PluginsErrorDictionary.h"
 
-#if ORTHANC_PLUGINS_ENABLED != 1
+#if !defined(ORTHANC_ENABLE_PLUGINS)
+#  error The macro ORTHANC_ENABLE_PLUGINS must be defined
+#endif
+
+
+#if ORTHANC_ENABLE_PLUGINS != 1
 
 #include <boost/noncopyable.hpp>
 
